module Solicit
  class ClaimPullRequest
    def initialize(url:, label:, assignee:, original:)
      @url      = url
      @label    = label
      @assignee = assignee
      @original = original
    end

    def self.perform(*args)
      new(*args).perform
    end

    def perform
      return unless claim_pull_request&.success?

      @original[:attachments].reject! { |a| a[:callback_id] == @url } unless needs_more_assignees?
      @original[:attachments] << { color: :good, text: claim_message }
      @original
    end

    private

    def needs_more_assignees?
      Solicit.needs_more_assignees.call(current_assignees, @label)
    end

    def claim_pull_request
      return unless @assignee['handle'].presence

      @claim_pull_request ||= HTTParty.post @url, default_github_params.merge(
        body: { assignees: Array(@assignee['handle']) }.to_json
      )
    end

    def current_assignees
      @current_assignees ||= HTTParty.get @url.gsub('/assignees', ''), default_github_params
    end

    def claim_message
      I18n.t :"engines.solicit.volunteer", claim_message_options
    end

    def claim_message_options
      {
        name: @assignee.fetch('name', 'Someone'),
        emoji: @assignee.fetch('emoji', 'star2'),
      }
    end

    def default_github_params
      { query: { access_token: Solicit.github_api_token } }
    end
  end
end
