module Solicit
  class PostToSlack
    attr_reader :text

    def initialize(url:, channel:, username:, payload:)
      @url      = url
      @channel  = channel
      @username = username
      @payload  = payload
      @text     = I18n.t(:"engines.solicit.title", title_options)
    end

    def self.perform(*args)
      new(*args).tap(&:perform)
    end

    def perform
      Slack::Notifier.new(@url, channel: @channel, username: @username).ping(
        text: text,
        attachments: [{
          title: I18n.t(:"engines.solicit.review"),
          callback_id: callback_id,
          color: :warning,
          actions: [claim_action],
        }]
      )
    end

    private

    def title_options
      {
        author: author_name,
        link:   @payload.dig(:pull_request, :html_url),
        title:  @payload.dig(:pull_request, :title),
        label:  @payload.dig(:label, :name),
      }
    end

    def callback_id
      [
        @payload.dig(:repository, :url),
        'issues',
        @payload.dig(:pull_request, :number),
        'assignees',
      ].join('/')
    end

    def claim_action
      {
        name: :claim,
        text: I18n.t(:"engines.solicit.claim"),
        type: :button,
        value: @payload.dig(:label, :name),
      }
    end

    def author_name
      Solicit.contributors_map.with_indifferent_access.detect do |_, value|
        value[:handle] == @payload.dig(:sender, :login)
      end&.dig(1, 'name') || I18n.t(:"engines.solicit.someone")
    end
  end
end
